@@ -7,7 +7,9 @@
 extern int gfarm_spool_server_listen_backlog;
 extern char *gfarm_spool_server_listen_address;
 extern int gfarm_spool_server_back_channel_rcvbuf_limit;
-extern char *gfarm_spool_root;
+#define GFARM_SPOOL_ROOT_NUM	5
+extern char *gfarm_spool_root[];
+gfarm_error_t parse_set_spool_root(char *);
 enum gfarm_spool_check_level {
 	GFARM_SPOOL_CHECK_LEVEL_DEFAULT,
 	GFARM_SPOOL_CHECK_LEVEL_DISABLE,
@@ -24,6 +26,7 @@ extern int gfarm_spool_digest_error_check;
 extern int gfarm_write_verify;
 extern int gfarm_write_verify_interval;
 extern int gfarm_write_verify_retry_interval;
+extern int gfarm_write_verify_log_interval;
 
 /* GFM dependent */
 enum gfarm_atime_type {
@@ -58,6 +61,7 @@ extern int gfarm_metadb_job_queue_length;
 extern int gfarm_metadb_heartbeat_interval;
 extern int gfarm_metadb_dbq_size;
 extern int gfarm_metadb_server_back_channel_sndbuf_limit;
+extern int gfarm_metadb_server_nfs_root_squash_support;
 extern int gfarm_metadb_replica_remover_by_host_sleep_time;
 extern int gfarm_metadb_replica_remover_by_host_inode_step;
 extern int gfarm_replica_check;
@@ -145,6 +149,8 @@ gfarm_error_t gfarm_config_type_by_name_for_metadb(const char *,
 	const struct gfarm_config_type **);
 char gfarm_config_type_get_format(const struct gfarm_config_type *);
 int gfarm_config_type_is_privileged_to_get(const struct gfarm_config_type *);
+gfarm_error_t gfarm_config_copyin(const struct gfarm_config_type *,
+	union gfarm_config_storage *);
 gfarm_error_t gfarm_config_copyout(const struct gfarm_config_type *,
 	union gfarm_config_storage *);
 
@@ -153,6 +159,7 @@ gfarm_error_t gfarm_config_local_name_to_string(const char *, char *, size_t);
 struct gfm_connection;
 gfarm_error_t gfm_client_config_name_to_string(struct gfm_connection *,
 	const char *, char *, size_t);
+gfarm_error_t gfm_client_config_set_by_string(struct gfm_connection *, char *);
 gfarm_error_t gfm_client_config_get_vars_request(struct gfm_connection *,
 	int, void **);
 gfarm_error_t gfm_client_config_get_vars_result(struct gfm_connection *,
