@@ -1,5 +1,5 @@
 /*
- * $Id: gfmd.c 10028 2016-01-12 13:39:51Z tatebe $
+ * $Id: gfmd.c 10144 2016-04-25 05:45:06Z tatebe $
  */
 
 #include <gfarm/gfarm_config.h>
@@ -438,6 +438,9 @@ protocol_switch(struct peer *peer, int from_client, int skip, int level,
 	case GFM_PROTO_CONFIG_GET:
 		e = gfm_server_config_get(peer, from_client, skip);
 		break;
+	case GFM_PROTO_CONFIG_SET:
+		e = gfm_server_config_set(peer, from_client, skip);
+		break;
 	case GFM_PROTO_REPLICA_LIST_BY_NAME:
 		e = gfm_server_replica_list_by_name(peer, from_client, skip);
 		break;
@@ -1710,6 +1713,14 @@ main(int argc, char **argv)
 	 */
 	write_pid();
 
+	/* after gflog_syslog_open/gfarm_daemon, but before pthread_create() */
+	if (!gfarm_metadb_server_nfs_root_squash_support) {
+		gflog_info(GFARM_MSG_1004491,
+		    "nfs_root_squash_support disabled");
+		gfarm_privilege_lock_disable();
+		gfarm_auth_root_squash_support_disable();
+	}
+
 	/*
 	 * We don't want SIGPIPE, but want EPIPE on write(2)/close(2).
 	 */
